﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media.Imaging;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Maps.MapControl;
using System.Collections.ObjectModel;
using Librarian.ServiceMap;
//using Microsoft.Phone.Controls;
using System.Linq;
using System.Device.Location;
using System.Diagnostics;

namespace Librarian.model
{
    public class MapManager
    {
        private GeocodeServiceClient svc;
        private Map myMap;
        private MapLayer layer;
        private GeoCoordinateWatcher watcher;
        private double myLat;
        private double myLon;
        bool myPositonAdded = false;

        public MapManager(Map map)
        {
            myMap = map;
            myMap.LogoVisibility = Visibility.Collapsed;
            myMap.CopyrightVisibility = Visibility.Collapsed;
            layer = new MapLayer();
            myMap.Children.Add(layer);

            watcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High); // using high accuracy
            watcher.MovementThreshold = 20; // use MovementThreshold to ignore noise in the signal
            watcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(watcher_StatusChanged);
            watcher.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(watcher_PositionChanged);
            watcher.Start();

            // instantiate Bing Maps GeocodeService
            svc = new GeocodeServiceClient("BasicHttpBinding_IGeocodeService");
            svc.GeocodeCompleted += (s, e) =>
            {
                var geoResult = (from r in e.Result.Results
                                 orderby (int)r.Confidence ascending
                                 select r).FirstOrDefault();
                if (geoResult != null)
                {
                    this.SetLocation(geoResult.Locations[0].Latitude,geoResult.Locations[0].Longitude,10,true);
                }
            };

        }


        public void setZoom(int zoom)
        {
            myMap.ZoomLevel = zoom;
        }

        public void centerMap(double lat, double lon)
        {
            myMap.Center = new Location(lat, lon);
        }

        
        public void centerOnLocation(String Address)
        {
            ServiceMap.GeocodeRequest request = new ServiceMap.GeocodeRequest();

            // Only accept results with high confidence.
            request.Options = new GeocodeOptions()
            {
                Filters = new ObservableCollection<FilterBase>
                {
                    new ConfidenceFilter()
                    {
                        MinimumConfidence = Confidence.High
                    }
                }
            };

            request.Credentials = new Credentials()
            {
                ApplicationId = "AsgvZlKkQu8mwlRSjse1kst3Y_fT4hJKm7zfgUesehOUdmBA6TWmHENy_JePvWb4"
            };

            request.Query = Address;

            // Make asynchronous call to fetch the geo coordinate data.
            svc.GeocodeAsync(request);
        }
        
        public void removeAllPushPin()
        {
            layer.Children.Clear();
        }

        public Pushpin addPushPin(double lat, double lon, Color c, string text = "")
        {
            Pushpin pin = new Pushpin()
            {
                Location = new Location(lat, lon)
            };

            pin.Background = new SolidColorBrush(c);
            pin.Content = text;
            pin.FontSize = 9;
            layer.Children.Add(pin);

            return pin;
        }

        public MapLayer addPushPinV2(double lat, double lon, Color c, string text = "")
        {

            MapLayer iconLayer = new MapLayer();
            iconLayer.VerticalAlignment = VerticalAlignment.Center;
            iconLayer.HorizontalAlignment = HorizontalAlignment.Center;

            Location location = new Location() { Latitude = lat, Longitude = lon };
            PositionOrigin position = PositionOrigin.Center;

            StackPanel panel = new StackPanel();
            panel.Orientation = Orientation.Vertical;
            panel.VerticalAlignment = VerticalAlignment.Center;
            panel.HorizontalAlignment = HorizontalAlignment.Center;
            panel.Width = 200;
            panel.Height = 150;


            ImageBrush ib = new ImageBrush();
            ib.ImageSource = new BitmapImage(new Uri("../images/libpin_blue.png", UriKind.RelativeOrAbsolute));
            if( c.Equals(Colors.Green) )
                ib.ImageSource = new BitmapImage(new Uri("../images/libpin_blue.png", UriKind.RelativeOrAbsolute));
            if (c.Equals(Colors.Yellow))
                ib.ImageSource = new BitmapImage(new Uri("../images/libpin_orange.png", UriKind.RelativeOrAbsolute));
            if (c.Equals(Colors.Red))
                ib.ImageSource = new BitmapImage(new Uri("../images/libpin_red.png", UriKind.RelativeOrAbsolute));

            panel.Background = ib;


            TextBlock textBlock = new TextBlock();
            textBlock.Text = "  " + text;
            textBlock.VerticalAlignment = VerticalAlignment.Stretch;
            textBlock.HorizontalAlignment = HorizontalAlignment.Center;
            textBlock.Width = 90;
            textBlock.Height = 70;
            textBlock.FontSize = 9;
            textBlock.Foreground = new SolidColorBrush(Colors.Black);
            //panel.Background = new SolidColorBrush(c);

            panel.Children.Add(textBlock);

            iconLayer.AddChild(panel, location, position);


            layer.Children.Add(iconLayer);

            return iconLayer;
        }

        /*
         * GPS
         * This methods are used only but this class
         */
        // Event handler for the GeoCoordinateWatcher.StatusChanged event.
        void watcher_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            switch (e.Status)
            {
                case GeoPositionStatus.Disabled:
                    // The Location Service is disabled or unsupported.
                    // Check to see whether the user has disabled the Location Service.
                    if (watcher.Permission == GeoPositionPermission.Denied)
                    {
                        // The user has disabled the Location Service on their device.
                        //statusTextBlock.Text = "you have this application access to location.";
                        Debug.WriteLine("you have this application access to location.");
                    }
                    else
                    {
                        //statusTextBlock.Text = "location is not functioning on this device";
                        Debug.WriteLine("location is not functioning on this device");
                    }
                    break;

                case GeoPositionStatus.Initializing:
                    // The Location Service is initializing.
                    // Disable the Start Location button.
                    break;

                case GeoPositionStatus.NoData:
                    // The Location Service is working, but it cannot get location data.
                    // Alert the user and enable the Stop Location button.
                    Debug.WriteLine("location data is not available.");
                    break;

                case GeoPositionStatus.Ready:
                    // The Location Service is working and is receiving location data.
                    // Show the current position and enable the Stop Location button.
                    Debug.WriteLine("location data is not available.");
                    break;
            }
        }

        void watcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            myLat = e.Position.Location.Latitude;
            myLon = e.Position.Location.Longitude;
            Debug.WriteLine("My Position is: "+myLat+", "+myLon);
            Pushpin currentPin = null;
            //MapLayer currentLayer = null;
            if (myPositonAdded)
            {
                layer.Children.Remove(currentPin);
                //layer.Children.Remove(currentLayer);
            }

            //currentLayer = addPushPinV2(this.myLat, this.myLon, Colors.Gray, "YOU");
            addPushPin(this.myLat, this.myLon, Colors.Red, "YOU");
            myPositonAdded = true;


            centerMap(this.myLat, this.myLon);
        }

        private void stopLocationButton_Click(object sender, RoutedEventArgs e)
        {
            watcher.Stop();
        }


        private void SetLocation(double latitude, double longitude, double zoomLevel, bool showLocator)
        {
            // Move the pushpin to geo coordinate
            Microsoft.Phone.Controls.Maps.Platform.Location location = new Microsoft.Phone.Controls.Maps.Platform.Location();
            location.Latitude = latitude;
            location.Longitude = longitude;
            //this.centerMap(latitude, longitude);
            this.addPushPin(latitude, longitude, Colors.Orange);
        }
         
    }
}
